#import <appkit/appkit.h>

@class DiagramCell,Post,Block,Connection,Socket,UHLabel,UHTool,UHToolbox;

@interface Diagram:View

{ 	
	NXEvent *currentEvent;
	List *blockList;
	List *cellList;
	List *connectionList;
	List *connectionsToSocket;
	List *itemList;
	List *orderedSelection;
	List *postList;
	List *selection;
	NXImage *bgImage;
	NXImage *background;
	NXColor bgColor;
	NXColor gridColor;
	NXRect dirtyRect;
	NXRect selectionRect;
	NXRect connectionRect;
	NXPoint oldMouse;
	NXPoint from;
	float minX,maxX,minY,maxY;
	float xMove,yMove;
	float dxRemainder;
	float dyRemainder;
	float pasteOffset;
	int grid;
	int mousedClass;
	int connectionState;
	int downState;
	int upState;
	int constraint;
	id mousedObject;
	id delegate;
	id connectionClass;
	id postClass;
	Socket *fromSocket;
	id inspectorManager;
	id rubberband;
	Post *proxyPost;
	Connection *proxyConnection;
	id connectionTool;
	Socket *selectedSocket;
	BOOL mouseDragged;
	BOOL somethingMoved;
	BOOL showGrid;
	BOOL snapToGrid;
	BOOL doubleClickFlag;
	BOOL notPullingWhenMouseDown;
	BOOL isLabelEditingEnabled;
	BOOL inSelection;
	BOOL selectionDrawMode;
	BOOL postsAllowed;
	BOOL connectionToBeDisallowed;
	BOOL twoPosts;
	BOOL connectionDrags;
	BOOL printMode;
	BOOL selectionIsModified;
}

//CLASS METHODS

+ findImageNamed:(const char *)imageName;
+ (UHToolbox *)initConnectionToolbox;
+ loadSocketImages;


//INSTANCE METHODS

- initFrame:(const NXRect *)rect;
- free;

- drawBackground;
- drawSelf:(NXRect *)rect :(int)c;
- (BOOL)printMode;

- setBackgroundImage:(NXImage *)anImage;
- (NXImage *)backgroundImage;
- setBackgroundColor:(NXColor)aColor;
- (NXColor)backgroundColor;
- setGridColor:(NXColor)aColor;
- (NXColor)gridColor;
- setGridInterval:(int)interval;
- (int)gridInterval;
- showGrid:(BOOL)flag;
- (BOOL)isGridShowing;
- setSnapOn:(BOOL)flag;
- (BOOL)isSnapOn;
- sizeTo:(NXCoord)width :(NXCoord)height;

- showItemInspector:sender;
- itemInspectorManager;
- setItemInspectorManager:anObject;
- objectWasInspected:object;
- objectListWasInspected:(List *)objectList;
- (const char*)getInspectorClassName;
- inspectSelection;

- alignSelectionToGrid:sender;
- pushItemBack:anItem;
- pushBack:sender;
- popItemForward:anItem;
- popForward:sender;
- sendItemToBack:anItem;
- sendToBack:sender;
- bringItemToFront:anItem;
- bringToFront:sender;
- moveCell:(DiagramCell *)cell toPoint:(NXPoint *)aPoint;

- setDelegate:anObject;
- delegate;

- setLabelEditing:(BOOL)flag;
- (BOOL)isLabelEditingEnabled;
- labelWasChanged:object;

- deleteAll:sender;
- cut:sender;
- delete:sender;
- copy:sender;
- paste:sender;
- select:item;
- selectList:(List *)aList;
- selectAll:sender;
- clearSelection:sender;
- selection;
- keyDown:(NXEvent *)event;

- (List *)cellList;
- (List *)blockList;
- (List *)postList;
- setPostClass:aClassObject;
- postClass;
- addPost:(Post *)post at:(float)x :(float)y;
- deletePost:(Post *)post;
- allowPosts:(BOOL)flag;
- (BOOL)postsAllowed;
- addBlock:(Block *)block at:(float)x :(float)y;
- deleteBlock:(Block *)block;


- setConnectionClass:aClassObject;
- connectionClass;
- (Connection *)newConnectionWithHead:(Socket *)socket1 andTail:(Socket *)socket2;   
- (Connection *)finishConnection:connection withHead:(Socket *)socket1 andTail:(Socket *)socket2;
- deleteConnection:(Connection *)aConnection;
- replace:oldObject withToolNamed:(char *)newToolName;
- (Connection *)replaceConnection:(Connection *)oldConn withToolNamed:(char *)newToolName;
- removeConnectionsTo:(DiagramCell *)aCell;
- (List *)connectionList;
- (List *)connectionsToSocket:(Socket *)socket;

- setConnectionTool:(UHTool *)tool;
- (UHTool *)connectionTool;

- toolbox:(UHToolbox *)sender selectedTool:aTool;

- writeDataToTypedStream:(NXTypedStream *)ts;
- readDataFromTypedStream:(NXTypedStream *)ts;
- findTools:(Storage *)names;

@end

@interface Object (DiagramDelegate)

//METHODS IMPLEMENTED BY THE DELEGATE
- diagram:(Diagram *)sender addedPost:(Post *)aPost; 
- (BOOL)diagram:(Diagram *)sender willDeletePost:(Post *)aPost;
- diagram:(Diagram *)sender deletedPost:(Post *)aPost;
- (BOOL)diagram:(Diagram *)sender willAddPostAtPoint:(NXPoint *)aPoint;
- diagram:(Diagram *)sender addedBlock:(Block *)aBlock;
- (BOOL)diagram:(Diagram *)sender willDeleteBlock:(Block *)aBlock;
- diagram:(Diagram *)sender deletedBlock:(Block *)aBlock; 
- diagram:(Diagram *)sender movedCell:(DiagramCell *)aCell; 
- diagram:(Diagram *)sender addedConnection:(Connection *)aConnection; 
- diagram:(Diagram *)sender deletedConnection:(Connection *)aConnection; 
- (BOOL)diagram:(Diagram *)sender willAddConnection:(Connection *)aConnection;
- (BOOL)diagram:(Diagram *)sender testingConnectionFrom:(Socket *)fromSock to:(Socket *)toSock;
- diagramWillDeleteAll:(Diagram *)sender;
- diagramDeletedAll:(Diagram *)sender; 
- diagram:(Diagram *)sender changedSelection:(List *)aSelection; 
- diagram:(Diagram *)sender doubleClickedItem:item; 
- diagram:(Diagram *)sender changedLabelOnItem:item;
- diagram:(Diagram *)sender needsTools:(Storage *)names;
- diagram:(Diagram *)sender replacedConnection:(Connection *)oldConn with:(Connection *)newConn;
@end